@@ -23,6 +23,8 @@ del getLogger
 _R = _hidpp10.REGISTERS
 _IR = _hidpp10.INFO_SUBREGISTERS
 
+KIND_MAP = {kind: _hidpp10.DEVICE_KIND[str(kind)] for kind in _hidpp20.DEVICE_KIND}
+
 #
 #
 #
@@ -38,11 +40,14 @@ class Device(object):
         self.receiver = receiver
         self.may_unpair = False
         self.isDevice = True  # some devices act as receiver so we need a property to distinguish them
+        self.handle = None
+        self.path = None
+        self.product_id = None
 
         if receiver:
             assert number > 0 and number <= receiver.max_devices
         else:
-            assert number == 0
+            assert number == 0xFF
         # Device number, 1..6 for unifying devices, 1 otherwise.
         self.number = number
         # 'device active' flag; requires manual management.
@@ -87,10 +92,6 @@ class Device(object):
         # See `add_notification_handler`
         self._notification_handlers = {}
 
-        self.handle = None
-        self.path = None
-        self.product_id = None
-
         # if _log.isEnabledFor(_DEBUG):
         #     _log.debug("new Device(%s, %s, %s)", receiver, number, link_notification)
 
@@ -197,6 +198,8 @@ class Device(object):
         if not self._codename:
             if self.online and self.protocol >= 2.0:
                 self._codename = _hidpp20.get_friendly_name(self)
+                if not self._codename:
+                    self._codename = self.name.split(' ', 1)[0] if self.name else None
             elif self.receiver:
                 codename = self.receiver.read_register(_R.receiver_info, _IR.device_name + self.number - 1)
                 if codename:
@@ -212,7 +215,7 @@ class Device(object):
         if not self._name:
             if self.online and self.protocol >= 2.0:
                 self._name = _hidpp20.get_name(self)
-        return self._name or self.codename or ('Unknown device %s' % (self.wpid or self.product_id))
+        return self._name or self._codename or ('Unknown device %s' % (self.wpid or self.product_id))
 
     @property
     def unitId(self):
@@ -256,7 +259,8 @@ class Device(object):
                 kind = ord(pair_info[7:8]) & 0x0F
                 self._kind = _hidpp10.DEVICE_KIND[kind]
             elif self.online and self.protocol >= 2.0:
-                self._kind = _hidpp20.get_kind(self)
+                kind = _hidpp20.get_kind(self)
+                self._kind = KIND_MAP[kind] if kind else None
         return self._kind or '?'
 
     @property
@@ -431,7 +435,6 @@ class Device(object):
         return None
 
     def request(self, request_id, *params, no_reply=False):
-
         if self:
             return _base.request(
                 self.handle or self.receiver.handle,
@@ -439,7 +442,8 @@ class Device(object):
                 request_id,
                 *params,
                 no_reply=no_reply,
-                long_message=self.bluetooth or self.protocol >= 2.0
+                long_message=self.bluetooth or self.protocol >= 2.0,
+                protocol=self.protocol
             )
 
     def feature_request(self, feature, function=0x00, *params, no_reply=False):
@@ -491,7 +495,7 @@ class Device(object):
         try:
             handle = _base.open_path(device_info.path)
             if handle:
-                return Device(None, 0, info=device_info)
+                return Device(None, 0xFF, info=device_info)
         except OSError as e:
             _log.exception('open %s', device_info)
             if e.errno == _errno.EACCES:
