@@ -38,17 +38,24 @@ _DUMMY = (
     _('empty'),
     _('critical'),
     _('low'),
+    _('average'),
     _('good'),
     _('full'),
 
     # battery charging statuses
     _('discharging'),
     _('recharging'),
+    _('charging'),
+    _('not charging'),
     _('almost full'),
     _('charged'),
     _('slow recharge'),
     _('invalid battery'),
     _('thermal error'),
+    _('error'),
+    _('standard'),
+    _('fast'),
+    _('slow'),
 
     # pairing errors
     _('device timeout'),
@@ -61,4 +68,28 @@ _DUMMY = (
     _('Bootloader'),
     _('Hardware'),
     _('Other'),
+
+    # common button and task names (from special_keys.py)
+    _('Left Button'),
+    _('Right Button'),
+    _('Middle Button'),
+    _('Back Button'),
+    _('Forward Button'),
+    _('Mouse Gesture Button'),
+    _('Smart Shift'),
+    _('DPI Switch'),
+    _('Left Tilt'),
+    _('Right Tilt'),
+    _('Left Click'),
+    _('Right Click'),
+    _('Mouse Middle Button'),
+    _('Mouse Back Button'),
+    _('Mouse Forward Button'),
+    _('Gesture Button Navigation'),
+    _('Mouse Scroll Left Button'),
+    _('Mouse Scroll Right Button'),
+
+    # key/button statuses
+    _('pressed'),
+    _('released'),
 )
