@@ -24,6 +24,9 @@ import math
 from copy import copy as _copy
 from logging import DEBUG as _DEBUG
 from logging import getLogger
+## use regular time instead of time_ns so as not to require Python 3.7
+# from time import time_ns as _time_ns
+from time import time as _time
 
 from . import hidpp20 as _hidpp20
 from . import special_keys as _special_keys
@@ -32,6 +35,7 @@ from .common import NamedInts as _NamedInts
 from .common import bytes2int as _bytes2int
 from .common import int2bytes as _int2bytes
 from .common import unpack as _unpack
+from .i18n import _
 
 _log = getLogger(__name__)
 del getLogger
@@ -519,7 +523,7 @@ class BitFieldWithOffsetAndMaskSetting(BitFieldSetting):
 class RegisterRW(object):
     __slots__ = ('register', )
 
-    kind = _NamedInt(0x01, 'register')
+    kind = _NamedInt(0x01, _('register'))
 
     def __init__(self, register):
         assert isinstance(register, int)
@@ -533,31 +537,35 @@ class RegisterRW(object):
 
 
 class FeatureRW(object):
-    __slots__ = ('feature', 'read_fnid', 'write_fnid', 'no_reply')
+    __slots__ = ('feature', 'read_fnid', 'write_fnid', 'prefix', 'no_reply')
 
-    kind = _NamedInt(0x02, 'feature')
+    kind = _NamedInt(0x02, _('feature'))
     default_read_fnid = 0x00
     default_write_fnid = 0x10
+    default_prefix = b''
 
-    def __init__(self, feature, read_fnid=default_read_fnid, write_fnid=default_write_fnid, no_reply=False):
+    def __init__(
+        self, feature, read_fnid=default_read_fnid, write_fnid=default_write_fnid, prefix=default_prefix, no_reply=False
+    ):
         assert isinstance(feature, _NamedInt)
         self.feature = feature
         self.read_fnid = read_fnid
         self.write_fnid = write_fnid
         self.no_reply = no_reply
+        self.prefix = prefix
 
     def read(self, device, data_bytes=b''):
         assert self.feature is not None
-        return device.feature_request(self.feature, self.read_fnid, data_bytes)
+        return device.feature_request(self.feature, self.read_fnid, self.prefix, data_bytes)
 
     def write(self, device, data_bytes):
         assert self.feature is not None
-        reply = device.feature_request(self.feature, self.write_fnid, data_bytes, no_reply=self.no_reply)
+        reply = device.feature_request(self.feature, self.write_fnid, self.prefix, data_bytes, no_reply=self.no_reply)
         return reply if not self.no_reply else True
 
 
 class FeatureRWMap(FeatureRW):
-    kind = _NamedInt(0x02, 'feature')
+    kind = _NamedInt(0x02, _('feature'))
     default_read_fnid = 0x00
     default_write_fnid = 0x10
     default_key_byte_count = 1
@@ -596,15 +604,16 @@ class FeatureRWMap(FeatureRW):
 
 
 class BooleanValidator(object):
-    __slots__ = ('true_value', 'false_value', 'mask', 'needs_current_value')
+    __slots__ = ('true_value', 'false_value', 'read_offset', 'mask', 'needs_current_value')
 
     kind = KIND.toggle
     default_true = 0x01
     default_false = 0x00
     # mask specifies all the affected bits in the value
     default_mask = 0xFF
+    default_read_offset = 0
 
-    def __init__(self, true_value=default_true, false_value=default_false, mask=default_mask):
+    def __init__(self, true_value=default_true, false_value=default_false, mask=default_mask, read_offset=default_read_offset):
         if isinstance(true_value, int):
             assert isinstance(false_value, int)
             if mask is None:
@@ -638,8 +647,10 @@ class BooleanValidator(object):
         self.true_value = true_value
         self.false_value = false_value
         self.mask = mask
+        self.read_offset = read_offset
 
     def validate_read(self, reply_bytes):
+        reply_bytes = reply_bytes[self.read_offset:]
         if isinstance(self.mask, int):
             reply_value = ord(reply_bytes[:1]) & self.mask
             if _log.isEnabledFor(_DEBUG):
@@ -682,6 +693,7 @@ class BooleanValidator(object):
                 to_write |= ord(current_value[:1]) & (0xFF ^ self.mask)
             if current_value is not None and to_write == ord(current_value[:1]):
                 return None
+            to_write = bytes([to_write])
         else:
             to_write = bytearray(to_write)
             count = len(self.mask)
@@ -721,7 +733,7 @@ class BitFieldValidator(object):
         r = _bytes2int(reply_bytes[:self.byte_count])
         value = {str(int(k)): False for k in self.options}
         m = 1
-        for _ in range(8 * self.byte_count):
+        for _ignore in range(8 * self.byte_count):
             if m in self.options:
                 value[str(int(m))] = bool(r & m)
             m <<= 1
@@ -795,7 +807,7 @@ class BitFieldWithOffsetAndMaskValidator(object):
             value = _bytes2int(b[:self.byte_count])
             mask_to_opt = self._option_from_offset_mask.get(offset, {})
             m = 1
-            for _ in range(8 * self.byte_count):
+            for _ignore in range(8 * self.byte_count):
                 if m in mask_to_opt:
                     values[str(int(mask_to_opt[m]))] = bool(value & m)
                 m <<= 1
@@ -1051,6 +1063,9 @@ class DivertedMouseMovement(object):
         self.dy = 0.
         self.fsmState = 'idle'
         self.dpiSetting = next(filter(lambda s: s.name == dpi_name, device.settings), None)
+        self.data = [0]
+        self.lastEv = 0.
+        self.skip = False
 
     @staticmethod
     def notification_handler(device, n):
@@ -1065,14 +1080,31 @@ class DivertedMouseMovement(object):
                 dx, dy = _unpack('!hh', n.data[:4])
                 state.handle_move_event(dx, dy)
 
+    def push_mouse_event(self):
+        x = int(self.dx)
+        y = int(self.dy)
+        if x == 0 and y == 0:
+            return
+        self.data.append(0)
+        self.data.append(x)
+        self.data.append(y)
+        self.data[0] += 1
+        self.dx = 0.
+        self.dy = 0.
+
     def handle_move_event(self, dx, dy):
         # This multiplier yields a more-or-less DPI-independent dx of about 5/cm
         # The multiplier could be configurable to allow adjusting dx
+        now = _time() * 1000  # _time_ns() / 1e6
         dpi = self.dpiSetting.read() if self.dpiSetting else 1000
         dx = float(dx) / float(dpi) * 15.
         self.dx += dx
         dy = float(dy) / float(dpi) * 15.
         self.dy += dy
+        if now - self.lastEv > 50. and not self.skip:
+            self.push_mouse_event()
+        self.lastEv = now
+        self.skip = False
         if self.fsmState == 'pressed':
             if abs(self.dx) >= 1. or abs(self.dy) >= 1.:
                 self.fsmState = 'moved'
@@ -1083,18 +1115,30 @@ class DivertedMouseMovement(object):
                 self.fsmState = 'pressed'
                 self.dx = 0.
                 self.dy = 0.
+                self.lastEv = _time() * 1000  # _time_ns() / 1e6
+                self.skip = True
         elif self.fsmState == 'pressed' or self.fsmState == 'moved':
             if self.key not in cids:
                 # emit mouse gesture notification
                 from .base import _HIDPP_Notification as _HIDPP_Notification
                 from .common import pack as _pack
                 from .diversion import process_notification as _process_notification
-                payload = _pack('!hh', int(self.dx), int(self.dy))
+                self.push_mouse_event()
+                payload = _pack('!' + (len(self.data) * 'h'), *self.data)
                 notification = _HIDPP_Notification(0, 0, 0, 0, payload)
                 _process_notification(self.device, self.device.status, notification, _hidpp20.FEATURE.MOUSE_GESTURE)
+                self.data.clear()
+                self.data.append(0)
                 self.fsmState = 'idle'
-                self.dx = 0.
-                self.dy = 0.
+            else:
+                last = (cids - {self.key, 0})
+                if len(last) != 0:
+                    self.push_mouse_event()
+                    self.data.append(1)
+                    self.data.append(list(last)[0])
+                    self.data[0] += 1
+                    self.lastEv = _time() * 1000  # _time_ns() / 1e6
+                    return True
 
 
 MouseGestureKeys = [
@@ -1120,7 +1164,9 @@ class DivertedMouseMovementRW(object):
                 state = device._divertedMMState
                 if n.address == 0x00:
                     cid1, cid2, cid3, cid4 = _unpack('!HHHH', n.data[:8])
-                    state.handle_keys_event({cid1, cid2, cid3, cid4})
+                    x = state.handle_keys_event({cid1, cid2, cid3, cid4})
+                    if x:
+                        return True
                 elif n.address == 0x10:
                     dx, dy = _unpack('!hh', n.data[:4])
                     state.handle_move_event(dx, dy)
