@@ -76,7 +76,7 @@ _SIDE_SCROLL = ('side-scroll', _('Side Scrolling'),
                   'instead of the standard side-scrolling events.'))
 _HI_RES_SCROLL = ('hi-res-scroll', _('Scroll Wheel High Resolution'),
                   _('High-sensitivity mode for vertical scroll with the wheel.'))
-_LOW_RES_SCROLL = ('lowres-smooth-scroll', _('Scroll Wheel HID++ Scrolling'),
+_LOW_RES_SCROLL = ('lowres-smooth-scroll', _('Scroll Wheel Diversion'),
                    _('HID++ mode for vertical scroll with the wheel.') + '\n' +
                    _('Effectively turns off wheel scrolling in Linux.'))
 _HIRES_INV = ('hires-smooth-invert', _('Scroll Wheel Direction'),
@@ -104,7 +104,7 @@ _DIVERT_KEYS = ('divert-keys', _('Key/Button Diversion'),
 _DISABLE_KEYS = ('disable-keyboard-keys', _('Disable keys'), _('Disable specific keyboard keys.'))
 _PLATFORM = ('multiplatform', _('Set OS'), _('Change keys to match OS.'))
 _CHANGE_HOST = ('change-host', _('Change Host'), _('Switch connection to a different host'))
-_THUMB_SCROLL_MODE = ('thumb-scroll-mode', _('Thumb Wheel HID++ Scrolling'),
+_THUMB_SCROLL_MODE = ('thumb-scroll-mode', _('Thumb Wheel Diversion'),
                       _('HID++ mode for horizontal scroll with the thumb wheel.') + '\n' +
                       _('Effectively turns off thumb scrolling in Linux.'))
 _THUMB_SCROLL_INVERT = ('thumb-scroll-invert', _('Thumb Wheel Direction'), _('Invert thumb wheel scroll direction.'))
@@ -186,9 +186,9 @@ _GESTURE2_GESTURES_LABELS = {
 
 _GESTURE2_PARAMS_LABELS = {
     _GP['ExtraCapabilities']: (None, None),  # not supported
-    _GP['PixelZone']: ('Pixel zone', None),  # TO DO: replace None with a short description
-    _GP['RatioZone']: ('Ratio zone', None),  # TO DO: replace None with a short description
-    _GP['ScaleFactor']: ('Scale factor', 'Sets the cursor speed.'),
+    _GP['PixelZone']: (_('Pixel zone'), None),  # TO DO: replace None with a short description
+    _GP['RatioZone']: (_('Ratio zone'), None),  # TO DO: replace None with a short description
+    _GP['ScaleFactor']: (_('Scale factor'), _('Sets the cursor speed.')),
 }
 
 _GESTURE2_PARAMS_LABELS_SUB = {
@@ -291,8 +291,8 @@ def _feature_new_fn_swap():
 # ignore the capabilities part of the feature - all devices should be able to swap Fn state
 # just use the current host (first byte = 0xFF) part of the feature to read and set the Fn state
 def _feature_k375s_fn_swap():
-    validator = _BooleanV(true_value=b'\xFF\x01', false_value=b'\xFF\x00')
-    return _Setting(_FN_SWAP, _FeatureRW(_F.K375S_FN_INVERSION), validator, device_kind=(_DK.keyboard, ))
+    validator = _BooleanV(true_value=b'\x01', false_value=b'\x00', read_offset=1)
+    return _Setting(_FN_SWAP, _FeatureRW(_F.K375S_FN_INVERSION, prefix=b'\xFF'), validator, device_kind=(_DK.keyboard, ))
 
 
 # FIXME: This will enable all supported backlight settings,
@@ -518,7 +518,7 @@ def _feature_dpi_sliding():
                     if 'raw XY' in key.flags and 'divertable' in key.flags and 'virtual' not in key.flags:
                         keys.append(key.key)
             if keys:
-                keys.insert(0, _NamedInt(0, 'Off'))
+                keys.insert(0, _NamedInt(0, _('Off')))
                 return _ChoicesV(_NamedInts.list(keys), byte_count=2)
 
     return _Setting(_DPI_SLIDING, _DpiSlidingRW(), callback=_feature_dpi_sliding_callback, device_kind=(_DK.mouse, ))
@@ -570,7 +570,7 @@ def _feature_mouse_gesture_callback(device):
                 if 'raw XY' in key.flags and 'divertable' in key.flags and 'virtual' not in key.flags:
                     keys.append(key.key)
         if keys:
-            keys.insert(0, _NamedInt(0, 'Off'))
+            keys.insert(0, _NamedInt(0, _('Off')))
             return _ChoicesV(_NamedInts.list(keys), byte_count=2)
 
 
@@ -664,7 +664,7 @@ def _feature_divert_keys_callback(device):
     choices = {}
     for k in device.keys:
         if 'divertable' in k.flags and 'virtual' not in k.flags:
-            choices[k.key] = [_NamedInt(0x00, 'Regular'), _NamedInt(0x01, 'Diverted')]
+            choices[k.key] = [_NamedInt(0x00, _('Regular')), _NamedInt(0x01, _('Diverted'))]
     if not choices:
         return None
     return _ChoicesMapV(choices, key_byte_count=2, byte_count=1, mask=0x01)
@@ -820,7 +820,7 @@ def _feature_divert_gkeys():
             super(_DivertGkeysRW, self).__init__(feature, write_fnid=0x20)
 
         def read(self, device):  # no way to read, so just assume not diverted
-            return 0x00
+            return b'\x00'
 
     rw = _DivertGkeysRW(_F.GKEY)
     return _Setting(_DIVERT_GKEYS, rw, _BooleanV(true_value=0x01, false_value=0x00, mask=0xff), device_kind=(_DK.keyboard, ))
