@@ -26,6 +26,8 @@ from logitech_receiver import settings_templates as _settings_templates
 from logitech_receiver.common import NamedInt as _NamedInt
 from logitech_receiver.common import strhex as _strhex
 
+_F = _hidpp20.FEATURE
+
 
 def _print_receiver(receiver):
     paired_count = receiver.count()
@@ -194,9 +196,11 @@ def _print_device(dev, num=None):
                 if wheel_status:
                     print('            Wheel Reports: %s' % wheel_status)
             elif feature == _hidpp20.FEATURE.NEW_FN_INVERSION:
-                inverted, default_inverted = _hidpp20.get_new_fn_inversion(dev)
-                print('            Fn-swap:', 'enabled' if inverted else 'disabled')
-                print('            Fn-swap default:', 'enabled' if default_inverted else 'disabled')
+                inversion = _hidpp20.get_new_fn_inversion(dev)
+                if inversion:
+                    inverted, default_inverted = inversion
+                    print('            Fn-swap:', 'enabled' if inverted else 'disabled')
+                    print('            Fn-swap default:', 'enabled' if default_inverted else 'disabled')
             elif feature == _hidpp20.FEATURE.HOSTS_INFO:
                 host_names = _hidpp20.get_host_names(dev)
                 for host, (paired, name) in host_names.items():
@@ -204,6 +208,8 @@ def _print_device(dev, num=None):
             elif feature == _hidpp20.FEATURE.DEVICE_NAME:
                 print('            Name: %s' % _hidpp20.get_name(dev))
                 print('            Kind: %s' % _hidpp20.get_kind(dev))
+            elif feature == _hidpp20.FEATURE.DEVICE_FRIENDLY_NAME:
+                print('            Friendly Name: %s' % _hidpp20.get_friendly_name(dev))
             elif feature == _hidpp20.FEATURE.DEVICE_FW_VERSION:
                 for fw in _hidpp20.get_firmware(dev):
                     extras = _strhex(fw.extras) if fw.extras else ''
@@ -222,7 +228,7 @@ def _print_device(dev, num=None):
                 else:
                     mode = 'On-Board'
                 print('            Device Mode: %s' % mode)
-            elif feature == _hidpp20.FEATURE.BATTERY_STATUS or feature == _hidpp20.FEATURE.BATTERY_VOLTAGE:
+            elif feature in (_F.BATTERY_STATUS, _F.BATTERY_VOLTAGE, _F.BATTERY_VOLTAGE):
                 print('', end='       ')
                 _battery_line(dev)
             for setting in dev_settings:
