@@ -22,13 +22,12 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 from logging import DEBUG as _DEBUG
 from logging import getLogger
 
-import gi
+import gi  # isort:skip
+gi.require_version('Gtk', '3.0')  # NOQA: E402
 
-from gi.repository import GLib, Gtk
-from logitech_receiver.status import ALERT
-from solaar.i18n import _
-
-gi.require_version('Gtk', '3.0')
+from gi.repository import GLib, Gtk  # NOQA: E402 # isort:skip
+from logitech_receiver.status import ALERT  # NOQA: E402 # isort:skip
+from solaar.i18n import _  # NOQA: E402 # isort:skip
 
 _log = getLogger(__name__)
 del getLogger
@@ -55,6 +54,12 @@ def _error_dialog(reason, object):
             _('Found a Logitech Receiver (%s), but did not have permission to open it.') % object + '\n\n' +
             _("If you've just installed Solaar, try removing the receiver and plugging it back in.")
         )
+    elif reason == 'nodevice':
+        title = _('Cannot connect to device error')
+        text = (
+            _('Found a Logitech receiver or device at %s, but encountered an error connecting to it.') % object + '\n\n' +
+            _('Try removing the device and plugging it back in or turning it off and then on.')
+        )
     elif reason == 'unpair':
         title = _('Unpairing failed')
         text = (
