@@ -27,6 +27,7 @@ from shlex import quote as shlex_quote
 from gi.repository import Gdk, GObject, Gtk
 from logitech_receiver import diversion as _DIV
 from logitech_receiver.diversion import XK_KEYS as _XK_KEYS
+from logitech_receiver.diversion import Key as _Key
 from logitech_receiver.diversion import buttons as _buttons
 from logitech_receiver.hidpp20 import FEATURE as _ALL_FEATURES
 from logitech_receiver.special_keys import CONTROL as _CONTROL
@@ -191,12 +192,12 @@ class DiversionDialog:
 
         button_box = Gtk.HBox(spacing=20)
         self.save_btn = Gtk.Button.new_from_icon_name('document-save', Gtk.IconSize.BUTTON)
-        self.save_btn.set_label('Save changes')
+        self.save_btn.set_label(_('Save changes'))
         self.save_btn.set_always_show_image(True)
         self.save_btn.set_sensitive(False)
         self.save_btn.set_valign(Gtk.Align.CENTER)
         self.discard_btn = Gtk.Button.new_from_icon_name('document-revert', Gtk.IconSize.BUTTON)
-        self.discard_btn.set_label('Discard changes')
+        self.discard_btn.set_label(_('Discard changes'))
         self.discard_btn.set_always_show_image(True)
         self.discard_btn.set_sensitive(False)
         self.discard_btn.set_valign(Gtk.Align.CENTER)
@@ -507,10 +508,12 @@ class DiversionDialog:
                     [
                         (_('Feature'), _DIV.Feature, FeatureUI.FEATURES_WITH_DIVERSION[0]),
                         (_('Process'), _DIV.Process, ''),
+                        (_('MouseProcess'), _DIV.MouseProcess, ''),
                         (_('Report'), _DIV.Report, 0),
                         (_('Modifiers'), _DIV.Modifiers, []),
                         (_('Key'), _DIV.Key, ''),
                         (_('Test'), _DIV.Test, next(iter(_DIV.TESTS))),
+                        (_('Mouse Gesture'), _DIV.MouseGesture, ''),
                     ]
                 ],
                 [
@@ -875,6 +878,34 @@ class ProcessUI(ConditionUI):
         return str(component.process)
 
 
+class MouseProcessUI(ConditionUI):
+
+    CLASS = _DIV.MouseProcess
+
+    def create_widgets(self):
+        self.widgets = {}
+        self.field = Gtk.Entry(halign=Gtk.Align.CENTER, valign=Gtk.Align.CENTER, hexpand=True, vexpand=True)
+        self.field.set_size_request(600, 0)
+        self.field.connect('changed', self._on_update)
+        self.widgets[self.field] = (0, 0, 1, 1)
+
+    def show(self, component):
+        super().show(component)
+        with self.ignore_changes():
+            self.field.set_text(component.process)
+
+    def collect_value(self):
+        return self.field.get_text()
+
+    @classmethod
+    def left_label(cls, component):
+        return _('MouseProcess')
+
+    @classmethod
+    def right_label(cls, component):
+        return str(component.process)
+
+
 class FeatureUI(ConditionUI):
 
     CLASS = _DIV.Feature
@@ -999,25 +1030,36 @@ class KeyUI(ConditionUI):
 
     def create_widgets(self):
         self.widgets = {}
-        self.field = CompletionEntry(
+        self.key_field = CompletionEntry(
             self.KEY_NAMES, halign=Gtk.Align.CENTER, valign=Gtk.Align.CENTER, hexpand=True, vexpand=True
         )
-        self.field.set_size_request(600, 0)
-        self.field.connect('changed', self._on_update)
-        self.widgets[self.field] = (0, 0, 1, 1)
+        self.key_field.set_size_request(600, 0)
+        self.key_field.connect('changed', self._on_update)
+        self.widgets[self.key_field] = (0, 0, 2, 1)
+        self.action_pressed_radio = Gtk.RadioButton.new_with_label_from_widget(None, _('Key down'))
+        self.action_pressed_radio.connect('toggled', self._on_update, _Key.DOWN)
+        self.widgets[self.action_pressed_radio] = (2, 0, 1, 1)
+        self.action_released_radio = Gtk.RadioButton.new_with_label_from_widget(self.action_pressed_radio, _('Key up'))
+        self.action_released_radio.connect('toggled', self._on_update, _Key.UP)
+        self.widgets[self.action_released_radio] = (3, 0, 1, 1)
 
     def show(self, component):
         super().show(component)
         with self.ignore_changes():
-            self.field.set_text(str(component.key) if self.component.key else '')
+            self.key_field.set_text(str(component.key) if self.component.key else '')
+            if not component.action or component.action == _Key.DOWN:
+                self.action_pressed_radio.set_active(True)
+            else:
+                self.action_released_radio.set_active(True)
 
     def collect_value(self):
-        return self.field.get_text()
+        action = _Key.UP if self.action_released_radio.get_active() else _Key.DOWN
+        return [self.key_field.get_text(), action]
 
     def _on_update(self, *args):
         super()._on_update(*args)
-        icon = 'dialog-warning' if not self.component.key else ''
-        self.field.set_icon_from_icon_name(Gtk.EntryIconPosition.SECONDARY, icon)
+        icon = 'dialog-warning' if not self.component.key or not self.component.action else ''
+        self.key_field.set_icon_from_icon_name(Gtk.EntryIconPosition.SECONDARY, icon)
 
     @classmethod
     def left_label(cls, component):
@@ -1025,7 +1067,7 @@ class KeyUI(ConditionUI):
 
     @classmethod
     def right_label(cls, component):
-        return '%s (%04X)' % (str(component.key), int(component.key)) if component.key else 'None'
+        return '%s (%04X) (%s)' % (str(component.key), int(component.key), _(component.action)) if component.key else 'None'
 
 
 class TestUI(ConditionUI):
@@ -1073,6 +1115,94 @@ class TestUI(ConditionUI):
         return str(component.test)
 
 
+class MouseGestureUI(ConditionUI):
+
+    CLASS = _DIV.MouseGesture
+    MOUSE_GESTURE_NAMES = [
+        'Mouse Up', 'Mouse Down', 'Mouse Left', 'Mouse Right', 'Mouse Up-left', 'Mouse Up-right', 'Mouse Down-left',
+        'Mouse Down-right'
+    ]
+    MOVE_NAMES = list(map(str, _CONTROL)) + MOUSE_GESTURE_NAMES
+
+    def create_widgets(self):
+        self.widgets = {}
+        self.fields = []
+        self.del_btns = []
+        self.add_btn = Gtk.Button(_('Add action'), halign=Gtk.Align.CENTER, valign=Gtk.Align.END, hexpand=True, vexpand=True)
+        self.add_btn.connect('clicked', self._clicked_add)
+        self.widgets[self.add_btn] = (1, 0, 1, 1)
+
+    def _create_field(self):
+        field = Gtk.ComboBoxText.new_with_entry()
+        for g in self.MOUSE_GESTURE_NAMES:
+            field.append(g, g)
+        CompletionEntry.add_completion_to_entry(field.get_child(), self.MOVE_NAMES)
+        field.connect('changed', self._on_update)
+        self.fields.append(field)
+        self.widgets[field] = (len(self.fields) - 1, 0, 1, 1)
+        return field
+
+    def _create_del_btn(self):
+        btn = Gtk.Button(_('Delete'), halign=Gtk.Align.CENTER, valign=Gtk.Align.START, hexpand=True, vexpand=True)
+        self.del_btns.append(btn)
+        self.widgets[btn] = (len(self.del_btns) - 1, 1, 1, 1)
+        btn.connect('clicked', self._clicked_del, len(self.del_btns) - 1)
+        return btn
+
+    def _clicked_add(self, _btn):
+        self.component.__init__(self.collect_value() + [''])
+        self.show(self.component)
+        self.fields[len(self.component.movements) - 1].grab_focus()
+
+    def _clicked_del(self, _btn, pos):
+        v = self.collect_value()
+        v.pop(pos)
+        self.component.__init__(v)
+        self.show(self.component)
+        self._on_update_callback()
+
+    def _on_update(self, *args):
+        super()._on_update(*args)
+        for i, f in enumerate(self.fields):
+            if f.get_visible():
+                icon = 'dialog-warning' if i < len(self.component.movements
+                                                   ) and self.component.movements[i] not in self.MOVE_NAMES else ''
+                f.get_child().set_icon_from_icon_name(Gtk.EntryIconPosition.SECONDARY, icon)
+
+    def show(self, component):
+        n = len(component.movements)
+        while len(self.fields) < n:
+            self._create_field()
+            self._create_del_btn()
+        self.widgets[self.add_btn] = (n + 1, 0, 1, 1)
+        super().show(component)
+        for i in range(n):
+            field = self.fields[i]
+            with self.ignore_changes():
+                field.get_child().set_text(component.movements[i])
+            field.set_size_request(int(0.3 * self.panel.get_toplevel().get_size()[0]), 0)
+            field.show_all()
+            self.del_btns[i].show()
+        for i in range(n, len(self.fields)):
+            self.fields[i].hide()
+            self.del_btns[i].hide()
+        self.add_btn.set_valign(Gtk.Align.END if n >= 1 else Gtk.Align.CENTER)
+
+    def collect_value(self):
+        return [f.get_active_text().strip() for f in self.fields if f.get_visible()]
+
+    @classmethod
+    def left_label(cls, component):
+        return _('Mouse Gesture')
+
+    @classmethod
+    def right_label(cls, component):
+        if len(component.movements) == 0:
+            return 'No-op'
+        else:
+            return ' -> '.join(component.movements)
+
+
 class ActionUI(RuleComponentUI):
 
     CLASS = _DIV.Action
@@ -1332,11 +1462,13 @@ COMPONENT_UI = {
     _DIV.Or: OrUI,
     _DIV.And: AndUI,
     _DIV.Process: ProcessUI,
+    _DIV.MouseProcess: MouseProcessUI,
     _DIV.Feature: FeatureUI,
     _DIV.Report: ReportUI,
     _DIV.Modifiers: ModifiersUI,
     _DIV.Key: KeyUI,
     _DIV.Test: TestUI,
+    _DIV.MouseGesture: MouseGestureUI,
     _DIV.KeyPress: KeyPressUI,
     _DIV.MouseScroll: MouseScrollUI,
     _DIV.MouseClick: MouseClickUI,
